/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3;

import java.io.PrintStream;

import org.xmloperator.lambda3.exception.IllegalFunctionArgumentException;
import org.xmloperator.lambda3.exception.ParsingException;
import org.xmloperator.lambda3.exception.UndefinedEliminationException;
import org.xmloperator.lambda3.exception.term.RedexSharingException;
import org.xmloperator.lambda3.exception.term.TermException;
import org.xmloperator.lambda3.exception.type.UntypableTermException;
import org.xmloperator.lambda3.impl.ParserImpl;
import org.xmloperator.lambda3.impl.TermReducerImpl;
import org.xmloperator.lambda3.impl.TermStatistic;
import org.xmloperator.lambda3.impl.TermWriterImpl;
import org.xmloperator.lambda3.impl.coding.BinaryNumberCoding;
import org.xmloperator.lambda3.impl.coding.ChurchNumberCoding;
import org.xmloperator.lambda3.impl.coding.FloatNumberCoding;
import org.xmloperator.lambda3.impl.coding.IntNumberCoding;
import org.xmloperator.lambda3.impl.coding.ScottNumberCoding;
import org.xmloperator.lambda3.impl.type.TermTypeFactory;
import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.Parser;
import org.xmloperator.lambda3.model.TermReducer;
import org.xmloperator.lambda3.model.TermWriter;
import org.xmloperator.lambda3.model.type.TermType;

/**
 * Test of LambdaAPI3.
 */
public abstract class Test {
	private static TermWriter termWriter = new TermWriterImpl();
	private static TermReducer termReducer = new TermReducerImpl();

	public static void main(String[] args) {
		PrintStream trace = System.out;
		try {
			long t1 = System.currentTimeMillis();
			allTests(trace);
			long t2 = System.currentTimeMillis();
			statistics(trace);
			trace.println("Total uses " + (t2 - t1) + " ms");
		} catch (ParsingException exception) {
			exception.printTrace(trace);
		} catch (IllegalFunctionArgumentException exception) {
			exception.printTrace(trace, termWriter);
		} catch (RedexSharingException exception) {
			exception.printTrace(trace, termWriter);
		} catch (TermException exception) {
			exception.printTrace(trace, termWriter);
		} catch (Exception exception) {
			exception.printStackTrace(trace);
		}
		System.exit(0);
	}

	private static void allTests(PrintStream trace) {
		testPrinter(trace);
		testType(trace);
		testReducer(trace);
		testCalculus(trace);
	}

	private static void testPrinter(PrintStream trace) {
		trace.println("TestPrinter with ParserImpl");
		{
			Parser parser = new ParserImpl();
			print(trace, "I", parser.parse("I"));
			print(trace, "K", parser.parse("K"));
			print(trace, "F", parser.parse("F"));
			print(trace, "S", parser.parse("S"));
			print(trace, "B", parser.parse("B"));
			print(trace, "C", parser.parse("C"));
			print(trace, "Y", parser.parse("Y"));
			print(trace, "D", parser.parse("\\x.xx"));
			print(trace, "O", parser.parse("(\\x.xx)(\\x.xx)"));
			print(trace, "V", parser.parse("\\xy.y((xx)y)"));
			print(trace, "T", parser.parse("(\\xy.y((xx)y))(\\xy.y((xx)y))"));
			print(trace, "N20", parser.parse("N20"));
			print(trace, "N33", parser.parse("N33"));
			print(trace, "[I,K]", parser.parse("\\x.xIK"));
			print(trace, "pair", parser.parse("\\xyz.zxy"));
			print(trace, "fst", parser.parse("\\x.xF"));
			print(trace, "snd", parser.parse("\\x.xK"));
			print(trace, "J", parser.parse("J"));
			print(trace, "J T", parser.parse("J T"));
			print(trace, "P", parser.parse("P"));
			print(trace, "P I", parser.parse("P I"));
			print(trace, "P T", parser.parse("P T"));
			print(trace, "T", parser.parse("T"));
			print(trace, "Z", parser.parse("Z"));
			print(trace, "[T,Z]", parser.parse("\\x.xTZ"));
			print(trace, "T P", parser.parse("T P"));
			print(trace, "E1", parser.parse("E1"));
			print(trace, "E1 P", parser.parse("E1 P"));
			print(trace, "E1 Z T", parser.parse("E1 Z T"));
			print(trace, "M (T Z T)", parser.parse("M (T Z T)"));
		}
		trace.println("TestPrinter with ChurchNumberCoding");
		{
			Parser parser = new ParserImpl();
			parser.addNumberCoding(ChurchNumberCoding.NUMBER_CODING);
			print(trace, "#isZero", parser.parse("#isZero"));
			print(trace, "#succ", parser.parse("#succ"));
			print(trace, "#pred", parser.parse("#pred"));
			print(trace, "#add", parser.parse("#add"));
			print(trace, "#mult", parser.parse("#mult"));
			print(trace, "#power", parser.parse("#power"));
			print(trace, "#1", parser.parse("#1"));
			print(trace, "#2", parser.parse("#2"));
		}
		trace.println("TestPrinter with ScottNumberCoding");
		{
			Parser parser = new ParserImpl();
			parser.addNumberCoding(ScottNumberCoding.NUMBER_CODING);
			print(trace, "@isZero", parser.parse("@isZero"));
			print(trace, "@succ", parser.parse("@succ"));
			print(trace, "@pred", parser.parse("@pred"));
			print(trace, "@1", parser.parse("@1"));
			print(trace, "@2", parser.parse("@2"));
			print(trace, "@fromChurch", parser.parse("@fromChurch"));
			print(trace, "@toChurch", parser.parse("@toChurch"));
		}
		trace.println("TestPrinter with BinaryNumberCoding");
		{
			Parser parser = new ParserImpl();
			parser.addNumberCoding(BinaryNumberCoding.NUMBER_CODING);
			print(trace, "$isZero", parser.parse("$isZero"));
			print(trace, "$succ", parser.parse("$succ"));
			print(trace, "$add", parser.parse("$add"));
			print(trace, "$mult", parser.parse("$mult"));
			print(trace, "$1", parser.parse("$1"));
			print(trace, "$2", parser.parse("$2"));
			print(trace, "$fromChurch", parser.parse("$fromChurch"));
			print(trace, "$toChurch", parser.parse("$toChurch"));
		}
		trace.println("TestPrinter with IntNumberCoding");
		{
			Parser parser = new ParserImpl();
			parser.addNumberCoding(IntNumberCoding.NUMBER_CODING);
			print(trace, "_isZero", parser.parse("_isZero"));
			print(trace, "_succ", parser.parse("_succ"));
			print(trace, "_pred", parser.parse("_pred"));
			print(trace, "_add", parser.parse("_add"));
			print(trace, "_sub", parser.parse("_sub"));
			print(trace, "_mult", parser.parse("_mult"));
			print(trace, "_div", parser.parse("_div"));
			print(trace, "_mod", parser.parse("_mod"));
			print(trace, "_1", parser.parse("_1"));
			print(trace, "_2", parser.parse("_2"));
			print(trace, "_add _2", parser.parse("_add _2"));
		}
		trace.println("TestPrinter with FloatNumberCoding");
		{
			Parser parser = new ParserImpl();
			parser.addNumberCoding(FloatNumberCoding.NUMBER_CODING);
			print(trace, "add", parser.parse("add"));
			print(trace, "sub", parser.parse("sub"));
			print(trace, "mult", parser.parse("mult"));
			print(trace, "div", parser.parse("div"));
			print(trace, "1", parser.parse("1"));
			print(trace, "2", parser.parse("2"));
			print(trace, "mult 2", parser.parse("mult 2"));
		}
	}

	private static void print(PrintStream trace, String identifier, Term term) {
		trace.println("	" + identifier);
		{
			TermType termType = null;
			try {
				termType = term.assignType();
			} catch (UntypableTermException exception) {
			}
			trace.println("		Type:      " + termType);
		}
		{
			String treeText = termWriter.lambdaText(term);
			trace.println("		Lambda:    " + treeText);
		}
		{
			String bruijnText = termWriter.indexedText(term);
			trace.println("		DeBruijn:  " + bruijnText);
		}
		{
			String graphText = termWriter.dagText(term);
			trace.println("		DAG:       " + graphText);
		}
		{
			String combinatorsText = null;
			try {
				Term combinatorsTerm = term.abstractionElimination(termReducer);
				combinatorsText = termWriter.dagText(combinatorsTerm);
				termReducer.breakTermDeeply(combinatorsTerm);
			} catch (UndefinedEliminationException e) {
			}
			trace.println("		Combinat:  " + combinatorsText);
		}
		termReducer.breakTermDeeply(term);
	}

	private static void testType(PrintStream trace) {
		trace.println("TestType");
		verifyType(trace, "#0", TermTypeFactory
				.newChurchIntTermType(TermTypeFactory.newVariableTermType()));
		verifyType(trace, "#1", TermTypeFactory
				.newChurchIntTermType(TermTypeFactory.newVariableTermType()));
		verifyType(trace, "#2", TermTypeFactory
				.newChurchIntTermType(TermTypeFactory.newVariableTermType()));
		verifyType(trace, "#succ", TermTypeFactory
				.newChurchIntToChurchIntTermType(TermTypeFactory
						.newVariableTermType(), TermTypeFactory
						.newVariableTermType()));
		verifyType(trace, "#pred", TermTypeFactory
				.newChurchIntToChurchIntTermType(TermTypeFactory
						.newVariableTermType(), TermTypeFactory
						.newVariableTermType()));
		verifyType(trace, "#add", TermTypeFactory
				.newChurchIntToChurchIntTermType(TermTypeFactory
						.newVariableTermType(), TermTypeFactory
						.newVariableTermType()));
		verifyType(trace, "#mult", TermTypeFactory
				.newChurchIntToChurchIntTermType(TermTypeFactory
						.newVariableTermType(), TermTypeFactory
						.newVariableTermType()));
		verifyType(trace, "#power", TermTypeFactory
				.newChurchIntToChurchIntTermType(TermTypeFactory
						.newVariableTermType(), TermTypeFactory
						.newVariableTermType()));
		verifyType(trace, "_0", TermTypeFactory.integerTermType);
		verifyType(trace, "_1", TermTypeFactory.integerTermType);
		verifyType(trace, "_2", TermTypeFactory.integerTermType);
		verifyType(trace, "_succ", TermTypeFactory.integerToIntegerTermType);
		verifyType(trace, "_pred", TermTypeFactory.integerToIntegerTermType);
		verifyType(trace, "_add",
				TermTypeFactory.integerToIntegerToIntegerTermType);
		verifyType(trace, "_mult",
				TermTypeFactory.integerToIntegerToIntegerTermType);
		verifyType(trace, "0", TermTypeFactory.floatTermType);
		verifyType(trace, "1", TermTypeFactory.floatTermType);
		verifyType(trace, "2", TermTypeFactory.floatTermType);
		verifyType(trace, "ln", TermTypeFactory.floatToFloatTermType);
		verifyType(trace, "exp", TermTypeFactory.floatToFloatTermType);
		verifyType(trace, "add",
				TermTypeFactory.floatToFloatToFloatTermType);
		verifyType(trace, "mult",
				TermTypeFactory.floatToFloatToFloatTermType);
	}

	private static void verifyType(PrintStream trace, String identifier,
			TermType referenceTermType) {
		Parser parser = new ParserImpl();
		parser.addNumberCoding(ChurchNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(IntNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(FloatNumberCoding.NUMBER_CODING);
		Term term = parser.parse(identifier);
		TermType termType = null;
		try {
			termType = term.assignType();
			termType.setEquivalence(referenceTermType);
		} catch (UntypableTermException exception) {
			termType = null;
		}
		trace.println("	" + identifier + " : " + termType);
		termReducer.breakTermDeeply(term);
	}

	private static void testReducer(PrintStream trace) {
		trace.println("TestReducer");
		Parser parser = new ParserImpl();
		parser.addNumberCoding(ChurchNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(ScottNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(BinaryNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(IntNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(FloatNumberCoding.NUMBER_CODING);
		reduce(trace, "II", parser.parse("II"));
		reduce(trace, "KKF", parser.parse("KKF"));
		reduce(trace, "FKF", parser.parse("FKF"));
		reduce(trace, "Combi[pair]", parser.parse("C(BB(BC(B(CI)I)))I"));
		reduce(trace, "#2 #2", parser.parse("#2 #2"));
		reduce(trace, "#4 #2", parser.parse("#4 #2"));
		reduce(trace, "#isZero #2", parser.parse("#isZero #2"));
		reduce(trace, "@isZero @2", parser.parse("@isZero @2"));
		reduce(trace, "$isZero $2", parser.parse("$isZero $2"));
		reduce(trace, "#Ssucc #-1", parser.parse("#Ssucc #-1"));
		reduce(trace, "@pred @3", parser.parse("@pred @3"));
		reduce(trace, "TKF", parser.parse("TKF"));
		reduce(trace, "ZKF", parser.parse("ZKF"));
		reduce(trace, "_isZero _2", parser.parse("_isZero _2"));
		reduce(trace, "_pred _3", parser.parse("_pred _3"));
		reduce(trace, "_add _2 _3", parser.parse("_add _2 _3"));
		reduce(trace, "_mult _3 _4", parser.parse("_mult _3 _4"));
		reduce(trace, "E1ZTT", parser.parse("E1ZTT"));
		reduce(trace, "mult 3 4", parser.parse("mult 3 4"));
		reduce(trace, "mult (mult 3 2) 4", parser.parse("mult (mult 3 2) 4"));
	}

	private static void reduce(PrintStream trace, String identifier, Term term) {
		trace.println("	" + identifier);
		ReductionReport reductionReport = new ReductionReport(trace,
				termWriter, 1000);
		term = termReducer.normalForm(term, false, reductionReport);
		termReducer.breakTermDeeply(term);
	}

	private static void testCalculus(PrintStream trace) {
		trace.println("TestCalculus");
		compute(trace, "II");
		compute(trace, "#isZero #2");
		compute(trace, "#succ #3");
		compute(trace, "#pred #5");
		compute(trace, "#add #5 #3");
		compute(trace, "#mult #5 #3");
		compute(trace, "#Ssucc #-7");
		compute(trace, "#Spred #+1");
		compute(trace, "#Spred #+0");
		compute(trace, "#Spred #-1");
		compute(trace, "#Smult #+5 #-3");
		compute(trace, "@isZero @2");
		compute(trace, "@succ @3");
		compute(trace, "@pred @4");
		compute(trace, "@Ssucc @-4");
		compute(trace, "@Spred @+1");
		compute(trace, "@Spred @+0");
		compute(trace, "@Spred @-1");
		compute(trace, "@fromChurch #2");
		compute(trace, "@toChurch @3");
		compute(trace, "$isZero $2");
		compute(trace, "$succ $3");
		compute(trace, "$pred $4");
		compute(trace, "$add $2 $3");
		compute(trace, "$mult $3 $3");
		compute(trace, "$fromChurch #2");
		compute(trace, "$toChurch $3");
		compute(trace, "#divByTwo #9");
		compute(trace, "#syracuse \\x.x#0K");
		compute(trace, "#syracuse \\x.x#1F");
		compute(trace, "#syracuseStepCount #1");
		compute(trace, "#syracuseStepCount #7");
		compute(trace, "$syracuse $1");
		compute(trace, "$syracuse $3");
		compute(trace, "$syracuseStepCount $1");
		compute(trace, "$syracuseStepCount $7");
		compute(trace, "_fromChurch #2");
		compute(trace, "_toChurch _3");
		compute(trace, "_fromBinary $2");
		compute(trace, "_toBinary _3");
		compute(trace, "_syracuse _7");
		compute(trace, "_syracuseStepCount _1");
		compute(trace, "_syracuseStepCount _7");
		compute(trace, "_syracuseStepCount _27");
		compute(trace, "_succ (_succ _7)");
		compute(trace, "mult 113 (mult 4 (atan 1))");
	}

	private static void compute(PrintStream trace, String text) {
		ReductionReport reductionReport = new ReductionReport(null, null,
				1000000);
		Parser parser = new ParserImpl();
		parser.addNumberCoding(ChurchNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(ScottNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(BinaryNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(IntNumberCoding.NUMBER_CODING);
		parser.addNumberCoding(FloatNumberCoding.NUMBER_CODING);
		Term term = parser.parse(text);
		long t1 = System.currentTimeMillis();
		term = termReducer.normalForm(term, false, reductionReport);
		long t2 = System.currentTimeMillis();
		trace.println("	" + text);
		trace.println("	 (" + reductionReport.getStepCount() + ") -> "
				+ termWriter.dagText(term) + " using " + (t2 - t1) + " ms");
		termReducer.breakTermDeeply(term);
	}

	private static void statistics(PrintStream trace) {
		trace.println("Statistics");

		trace.println("	Variables");
		trace.println("		free     : " + TermStatistic.getFreeVariableCount());
		trace.println("		living   : " + TermStatistic.getLivingVariableCount());
		trace.println("		consumed : " + TermStatistic.getConsumedVariableCount());
		trace.println("		reusable : "
				+ TermStatistic.getReusableVariableStockNumber());

		trace.println("	Constants(2)");
		trace.println("		living   : " + TermStatistic.getLivingConstant2Count());
		trace
				.println("		consumed : "
						+ TermStatistic.getConsumedConstant2Count());
		trace.println("		reusable : "
				+ TermStatistic.getReusableConstant2StockNumber());

		trace.println("	Constants(3)");
		trace.println("		living   : " + TermStatistic.getLivingConstant3Count());
		trace
				.println("		consumed : "
						+ TermStatistic.getConsumedConstant3Count());
		trace.println("		reusable : "
				+ TermStatistic.getReusableConstant3StockNumber());

		trace.println("	Abstractions");
		trace
				.println("		living   : "
						+ TermStatistic.getLivingAbstractionCount());
		trace.println("		consumed : "
				+ TermStatistic.getConsumedAbstractionCount());
		trace.println("		reusable : "
				+ TermStatistic.getReusableAbstractionStockNumber());

		trace.println("	Applications");
		trace
				.println("		living   : "
						+ TermStatistic.getLivingApplicationCount());
		trace.println("		consumed : "
				+ TermStatistic.getConsumedApplicationCount());
		trace.println("		reusable : "
				+ TermStatistic.getReusableApplicationStockNumber());
	}
}
