/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.exception;

import java.io.PrintStream;

import org.xmloperator.lambda3.model.Function;
import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.TermWriter;

/**
 * Exception raised on trying to apply a wrong argument to a constant function.
 */
public class IllegalFunctionArgumentException extends RuntimeException {
	private static final long serialVersionUID = -6218867326211465088L;

	private Function function;
	private Term arg1, arg2, arg3;

	public IllegalFunctionArgumentException(Function function, Term arg1) {
		this(function, arg1, null, null);
	}

	public IllegalFunctionArgumentException(Function function, Term arg1,
			Term arg2) {
		this(function, arg1, arg2, null);
		if (arg2 == null)
			throw new IllegalArgumentException();
	}

	public IllegalFunctionArgumentException(Function function, Term arg1,
			Term arg2, Term arg3) {
		if (function == null || arg1 == null)
			throw new IllegalArgumentException();
		this.function = function;
		this.arg1 = arg1;
		this.arg2 = arg2;
		this.arg3 = arg3;
	}

	public void printTrace(PrintStream trace, TermWriter termWriter) {
		trace.println(this.getClass().getName());
		trace.println("Function: " + this.function.getName());
		trace.println("Argument1: " + termWriter.dagText(this.arg1));
		if (this.arg2 != null)
			trace.println("Argument2: " + termWriter.dagText(this.arg2));
		if (this.arg3 != null)
			trace.println("Argument3: " + termWriter.dagText(this.arg3));
		this.printStackTrace(trace);
	}
}
