/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.impl;

import java.io.IOException;
import java.io.Reader;

import org.xmloperator.lambda3.exception.ParsingException;

/**
 * Term parsing at the lexical level.
 */
final class ParsingInput {
	private Reader in;
	private int ch;
	private int location = -1;

	public ParsingInput(Reader in) throws IOException {
		this.in = in;
		this.next();
	}

	public boolean isEmpty() {
		return this.ch < 0;
	}

	public char getChar() {
		return (char) this.ch;
	}

	public int getLocation() {
		return this.location;
	}

	public void next() throws IOException {
		this.ch = this.in.read();
		this.location++;
	}

	public boolean isChar(char expectedChar) {
		return this.ch == expectedChar;
	}

	public boolean isNumber() {
		return this.ch >= '0' && this.ch <= '9';
	}

	public boolean isVariableIdentifier() {
		return this.ch >= 'a' && this.ch <= 'z';
	}

	public boolean isExtensionIdentifier() {
		return (this.ch >= 'A' && this.ch <= 'Z')
				|| (this.ch >= 'a' && this.ch <= 'z');
	}

	public void throwsParsingException(String message) {
		throw new ParsingException(this.location, message);
	}

	public void parseChar(char expectedChar) throws IOException {
		if (this.ch != expectedChar)
			this.throwsParsingException("'" + expectedChar + "' expected");
		this.next();
	}

	public int parseNumber() throws IOException {
		if (!this.isNumber())
			this.throwsParsingException("Number expected");
		int number = this.ch - '0';
		this.next();
		while (this.isNumber()) {
			number = 10 * number + (ch - '0');
			this.next();
		}
		return number;
	}

	public String parseVariableIdentifier() throws IOException {
		if (!this.isVariableIdentifier())
			this.throwsParsingException("Variable identifier expected");
		StringBuffer sb = new StringBuffer();
		sb.append((char) this.ch);
		this.next();
		while (this.isNumber()) {
			sb.append((char) this.ch);
			this.next();
		}
		return sb.toString();
	}

	public String parseExtensionIdentifier() throws IOException {
		if (!this.isExtensionIdentifier())
			this.throwsParsingException("Extension identifier expected");
		StringBuffer sb = new StringBuffer();
		sb.append((char) this.ch);
		this.next();
		while (this.isExtensionIdentifier()) {
			sb.append((char) this.ch);
			this.next();
		}
		return sb.toString();
	}
}
