/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.impl.type;

import java.util.List;

import org.xmloperator.lambda3.model.type.FunctionTermType;
import org.xmloperator.lambda3.model.type.TermType;

final class FunctionTermTypeImpl extends TermTypeImpl implements
		FunctionTermType {
	final TermType input;
	final TermType output;

	FunctionTermTypeImpl(TermType input, TermType output) {
		if (input == null | output == null)
			throw new IllegalArgumentException();
		this.input = input;
		this.output = output;
	}

	// ////////////////////////
	// TermType implementation
	// ////////////////////////

	public boolean isFunctionTermType() {
		return true;
	}

	public void setEquivalence(TermType replacement) {
		if (replacement == this)
			return;
		replacement = replacement.getTerminalType();
		if (replacement.isFunctionTermType()) {
			FunctionTermTypeImpl function = ((FunctionTermTypeImpl) replacement);
			this.input.getTerminalType().setEquivalence(function.input);
			this.output.getTerminalType().setEquivalence(function.output);
		} else
			replacement.setEquivalence(this);
	}

	public boolean isUsing(TermType termType) {
		return this.input.getTerminalType().isUsing(termType)
				|| this.output.getTerminalType().isUsing(termType);
	}

	// ////////////////////////////////
	// FunctionTermType implementation
	// ////////////////////////////////

	public TermType getInput() {
		return this.input;
	}

	public TermType getOutput() {
		return this.output;
	}

	// //////////////////////
	// TermTypeImpl override
	// //////////////////////

	void toString(StringBuffer sb, List<TermType> terminalList) {
		boolean isInputParenthesized = this.input.getTerminalType()
				.isFunctionTermType();
		if (isInputParenthesized)
			sb.append(TermType.CHAR_OPENED_PARENTHESIS);
		((TermTypeImpl) this.input).toString(sb, terminalList);
		if (isInputParenthesized)
			sb.append(TermType.CHAR_CLOSED_PARENTHESIS);
		sb.append(TermType.TEXT_RIGHT_ARROW);
		((TermTypeImpl) this.output).toString(sb, terminalList);
	}
}
