/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.model;

import java.io.IOException;
import java.io.Reader;

/**
 * Term parser.
 */
public interface Parser {

	/**
	 * Adds a NumberCoding to this Parser.
	 * 
	 * @param numberCoding
	 *            a NumberCoding.
	 */
	public void addNumberCoding(NumberCoding numberCoding);

	/**
	 * Parses a closed Term from a Reader, using the following syntax :
	 * 
	 * <pre>
	 * Term ::= (' ')* (Variable | Abstraction | Application | '(' Term ')' | Predefined | Extension)
	 * Variable ::= Lowercase Number?
	 * Lowercase ::= ('a' | ... | 'z')
	 * Number ::= ('0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9')
	 * Abstraction ::= '\' Variable+ '.' Term
	 * Application ::= Term Term
	 * Predefined ::= 'I' | 'K' | 'F' | 'S' | 'B' | 'C' | 'Y' | ('N' Number Number)
	 * 					| 'J' | 'P' | 'T' | 'Z' | 'E1' | 'E2' | 'E3' 
	 * Extension ::= ('_' | '#' | '@' | '$') (Identifier | (('+' | '-')? Number+)) (' ')?
	 * Identifier ::= (Uppercase | LowerCase)*
	 * Uppercase ::= ('A' | ... | 'Z')
	 * </pre>
	 * 
	 * @param reader
	 *            a Reader.
	 * @return the parsed closed Term.
	 * @throws ParsingException
	 *             in case of parsing error.
	 */
	public Term parse(Reader reader) throws IOException;

	/**
	 * Parses a closed Term from a String, using the syntax given above.
	 * 
	 * @param text
	 *            a Term String representation.
	 * @return the parsed closed Term.
	 * @throws ParsingException
	 *             in case of parsing error.
	 */
	public Term parse(String text);
}
