/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.model;

/**
 * Tools for term printing.
 */
public interface TermPrinter {
	/*
	 * Lambda Term printing syntax elements.
	 */
	public static final char CHAR_LAMBDA = '\\';
	public static final char CHAR_POINT = '.';
	public static final char CHAR_BOUND_VARIABLE = 'x';
	public static final char CHAR_EXTERNAL_BOUND_VARIABLE = 'z';
	public static final char CHAR_UNNAMED_FREE_VARIABLE = '?';
	public static final char CHAR_BROKEN_REFERENCE = '!';
	public static final char CHAR_OPENED_PARENTHESIS = '(';
	public static final char CHAR_CLOSED_PARENTHESIS = ')';
	public static final char CHAR_UNDERSCORE = '_';
	public static final char CHAR_HASH = '#';
	public static final char CHAR_AT = '@';
	public static final char CHAR_DOLLAR = '$';
	public static final char CHAR_LIVRE = '';
	public static final char CHAR_ONE_OF_N = 'N';
	public static final char CHAR_SPACE = ' ';
	public static final char CHAR_PLUS = '+';
	public static final char CHAR_MINUS = '-';
	public static final char CHAR_DASH = '-';
	public static final char CHAR_STAR = '*';
	public static final char CHAR_SHARED_TERM = 'A';
	public static final char CHAR_LEFT_BRACKET = '[';
	public static final char CHAR_COMMA = ',';
	public static final char CHAR_RIGTH_BRACKET = ']';
	public static final String CHAR_SEPARATOR = "  ";
	public static final char CHAR_EQUIVALENCE = '=';

	/*
	 * Predefined lambda Terms naming.
	 */
	public static final char CHAR_I = 'I';
	public static final char CHAR_K = 'K';
	public static final char CHAR_F = 'F';
	public static final char CHAR_S = 'S';
	public static final char CHAR_B = 'B';
	public static final char CHAR_C = 'C';
	public static final char CHAR_Y = 'Y';
	public static final char CHAR_J = 'J';
	public static final char CHAR_P = 'P';
	public static final char CHAR_T = 'T';
	public static final char CHAR_Z = 'Z';
	public static final char CHAR_E = 'E';
	public static final char CHAR_M = 'M';

	/**
	 * Prints a root Term using the indexed notation of De Bruijn.
	 * 
	 * <pre>
	 * Example : "\1".
	 * </pre>
	 * 
	 * @param rootTerm
	 *            a root Term.
	 */
	public void printAsIndexed(Term rootTerm);

	/**
	 * Prints a Term as usual lambda term. If the underlinedTerm argument is not
	 * null then prints an underscore of the underlinedTerm.
	 * 
	 * <pre>
	 * Example : "\x1.x1".
	 * </pre>
	 * 
	 * @param rootTerm
	 *            a root Term.
	 */
	public void printAsLambda(Term rootTerm);

	/**
	 * Some terms have predefined identifier. This method returns the predefined
	 * identifier for a root Term if any, 0 elsewhere.
	 * <p>
	 * List of predefined terms :
	 * </p>
	 * 
	 * <pre>
	 * I = \x.x
	 * K = \xy.x
	 * F = \xy.y
	 * S = \xyz.xz(yz)
	 * B = \xyz.x(yz)
	 * C = \xyz.xzy
	 * Y = \x.(\y.x(yy))(\y.x(yy))
	 * </pre>
	 * 
	 * @param rootTerm
	 *            a root Term.
	 * @return the predefined identifier associated to the given root Term, if
	 *         any, 0 elsewhere.
	 */
	public char predefinedIdentifier(Term rootTerm);

	/**
	 * Prints a root Term as a DAG. if the underlinedTerm argument is not null
	 * then prints an underscore of the underlinedTerm.
	 * 
	 * <pre>
	 * Example(Omega): "A1A1 A1=\x1.x1x1".
	 * </pre>
	 * 
	 * @param rootTerm
	 *            a root Term.
	 */
	public void printAsDAG(Term rootTerm);
}
